<?php
/** 
 * ValidaCPFCNPJ valida e formata CPF e CNPJ
 *
 * Exemplo de uso:
 * $cpf_cnpj  = new ValidaCPFCNPJ('71569042000196');
 * $formatado = $cpf_cnpj->formata(); // 71.569.042/0001-96
 * $valida    = $cpf_cnpj->valida(); // True -> Vlido
 *
 * @package  valida-cpf-cnpj
 * @author   Luiz Otvio Miranda <contato@tutsup.com>
 * @version  v1.3
 * @access   public
 * @see      http://www.tutsup.com/
 */
class ValidaCPFCNPJ
{
	/** 
	 * Configura o valor (Construtor)
	 * 
	 * Remove caracteres invlidos do CPF ou CNPJ
	 * 
	 * @param string $valor - O CPF ou CNPJ
	 */
	function __construct ( $valor = null ) {
		// Deixa apenas nmeros no valor
		$this->valor = preg_replace( '/[^0-9]/', '', $valor );
		
		// Garante que o valor  uma string
		$this->valor = (string)$this->valor;
	}

	/**
	 * Verifica se  CPF ou CNPJ
	 * 
	 * Se for CPF tem 11 caracteres, CNPJ tem 14
	 * 
	 * @access protected
	 * @return string CPF, CNPJ ou false
	 */
	protected function verifica_cpf_cnpj () {
		// Verifica CPF
		if ( strlen( $this->valor ) === 11 ) {
			return 'CPF';
		} 
		// Verifica CNPJ
		elseif ( strlen( $this->valor ) === 14 ) {
			return 'CNPJ';
		} 
		// No retorna nada
		else {
			return false;
		}
	}

	/**
	 * Multiplica dgitos vezes posies
	 *
	 * @access protected
	 * @param  string    $digitos      Os digitos desejados
	 * @param  int       $posicoes     A posio que vai iniciar a regresso
	 * @param  int       $soma_digitos A soma das multiplicaes entre posies e dgitos
	 * @return int                     Os dgitos enviados concatenados com o ltimo dgito
	 */
	protected function calc_digitos_posicoes( $digitos, $posicoes = 10, $soma_digitos = 0 ) {
		// Faz a soma dos dgitos com a posio
		// Ex. para 10 posies:
		//   0    2    5    4    6    2    8    8   4
		// x10   x9   x8   x7   x6   x5   x4   x3  x2
		//   0 + 18 + 40 + 28 + 36 + 10 + 32 + 24 + 8 = 196
		for ( $i = 0; $i < strlen( $digitos ); $i++  ) {
			// Preenche a soma com o dgito vezes a posio
			$soma_digitos = $soma_digitos + ( $digitos[$i] * $posicoes );

			// Subtrai 1 da posio
			$posicoes--;

			// Parte especfica para CNPJ
			// Ex.: 5-4-3-2-9-8-7-6-5-4-3-2
			if ( $posicoes < 2 ) {
				// Retorno a posio para 9
				$posicoes = 9;
			}
		}

		// Captura o resto da diviso entre $soma_digitos dividido por 11
		// Ex.: 196 % 11 = 9
		$soma_digitos = $soma_digitos % 11;

		// Verifica se $soma_digitos  menor que 2
		if ( $soma_digitos < 2 ) {
			// $soma_digitos agora ser zero
			$soma_digitos = 0;
		} else {
			// Se for maior que 2, o resultado  11 menos $soma_digitos
			// Ex.: 11 - 9 = 2
			// Nosso dgito procurado  2
			$soma_digitos = 11 - $soma_digitos;
		}

		// Concatena mais um dgito aos primeiro nove dgitos
		// Ex.: 025462884 + 2 = 0254628842
		$cpf = $digitos . $soma_digitos;

		// Retorna
		return $cpf;
	}

	/**
	 * Valida CPF
	 *
	 * @author                Luiz Otvio Miranda <contato@tutsup.com>
	 * @access protected
	 * @param  string    $cpf O CPF com ou sem pontos e trao
	 * @return bool           True para CPF correto - False para CPF incorreto
	 */
	protected function valida_cpf() {
		// Captura os 9 primeiros dgitos do CPF
		// Ex.: 02546288423 = 025462884
		$digitos = substr($this->valor, 0, 9);

		// Faz o clculo dos 9 primeiros dgitos do CPF para obter o primeiro dgito
		$novo_cpf = $this->calc_digitos_posicoes( $digitos );

		// Faz o clculo dos 10 dgitos do CPF para obter o ltimo dgito
		$novo_cpf = $this->calc_digitos_posicoes( $novo_cpf, 11 );

		// Verifica se o novo CPF gerado  idntico ao CPF enviado
		if ( $novo_cpf === $this->valor ) {
			// CPF vlido
			return true;
		} else {
			// CPF invlido
			return false;
		}
	}

	/**
	 * Valida CNPJ
	 *
	 * @author                  Luiz Otvio Miranda <contato@tutsup.com>
	 * @access protected
	 * @param  string     $cnpj
	 * @return bool             true para CNPJ correto
	 */
	protected function valida_cnpj () {
		// O valor original
		$cnpj_original = $this->valor;

		// Captura os primeiros 12 nmeros do CNPJ
		$primeiros_numeros_cnpj = substr( $this->valor, 0, 12 );

		// Faz o primeiro clculo
		$primeiro_calculo = $this->calc_digitos_posicoes( $primeiros_numeros_cnpj, 5 );

		// O segundo clculo  a mesma coisa do primeiro, porm, comea na posio 6
		$segundo_calculo = $this->calc_digitos_posicoes( $primeiro_calculo, 6 );

		// Concatena o segundo dgito ao CNPJ
		$cnpj = $segundo_calculo;

		// Verifica se o CNPJ gerado  idntico ao enviado
		if ( $cnpj === $cnpj_original ) {
			return true;
		}
	}

	/**
	 * Valida
	 * 
	 * Valida o CPF ou CNPJ
	 * 
	 * @access public
	 * @return bool      True para vlido, false para invlido
	 */
	public function valida () {
		// Valida CPF
		if ( $this->verifica_cpf_cnpj() === 'CPF' ) {
			// Retorna true para cpf vlido
			return $this->valida_cpf();
		} 
		// Valida CNPJ
		elseif ( $this->verifica_cpf_cnpj() === 'CNPJ' ) {
			// Retorna true para CNPJ vlido
			return $this->valida_cnpj();
		} 
		// No retorna nada
		else {
			return false;
		}
	}

	/**
	 * Formata CPF ou CNPJ
	 *
	 * @access public
	 * @return string  CPF ou CNPJ formatado
	 */
	public function formata() {
		// O valor formatado
		$formatado = false;

		// Valida CPF
		if ( $this->verifica_cpf_cnpj() === 'CPF' ) {
			// Verifica se o CPF  vlido
			if ( $this->valida_cpf() ) {
				// Formata o CPF ###.###.###-##
				$formatado  = substr( $this->valor, 0, 3 ) . '.';
				$formatado .= substr( $this->valor, 3, 3 ) . '.';
				$formatado .= substr( $this->valor, 6, 3 ) . '-';
				$formatado .= substr( $this->valor, 9, 2 ) . '';
			}
		} 
		// Valida CNPJ
		elseif ( $this->verifica_cpf_cnpj() === 'CNPJ' ) {
			// Verifica se o CPF  vlido
			if ( $this->valida_cnpj() ) {
				// Formata o CNPJ ##.###.###/####-##
				$formatado  = substr( $this->valor,  0,  2 ) . '.';
				$formatado .= substr( $this->valor,  2,  3 ) . '.';
				$formatado .= substr( $this->valor,  5,  3 ) . '/';
				$formatado .= substr( $this->valor,  8,  4 ) . '-';
				$formatado .= substr( $this->valor, 12, 14 ) . '';
			}
		} 

		// Retorna o valor 
		return $formatado;
	}
}
?>